import 'package:flutter/material.dart';
import 'package:mui/mui.dart';

class ModalPage extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('Modal')
      ),
      body: Padding(
        padding: EdgeInsets.all(10.0),
        child: ListView(children: <Widget>[
          OwnCard(
            title: '简介',
            content: ['Modal类定义了一个show方法，打开一个向用户传递信息的弹出层']
          ),
          SizedBox(height: 20.0),
          OwnCard(
            title: 'API',
            content: ['Modal.show(ModalData data)',
              'ModalData:{',
              '   BuildContext context: 上下文，必传',
              '   String text: 弹窗内容',
              '   double width: 弹窗宽度',
              '   bool maskClosable: 点击蒙层是否允许关闭',
              '   bool maskClosable: 点击蒙层是否允许关闭，默认true',
              '   String okText: 确认按钮文字',
              '   String cancelText: 取消按钮文字',
              '   Function onCancel: 点击取消按钮回调',
              '   Function onOk: 点击确认按钮回调',
              '}'
            ]
          ),
          Container(
            margin: EdgeInsets.symmetric(vertical: 20.0),
            child: RaisedButton(
              child: Text("点我"),
              onPressed: () {
                Modal.show(ModalData(
                  context: context,
                  text: '我是一个弹窗',
                  okText: '确认',
                  cancelText: '取消',
                  onOk: () {
                    print('点击了确认');
                  }
                ));
              }
            )
          ),
          SizedBox(height: 20.0),
          OwnCard(
            title: 'demo',
            content: ['Modal.show(ModalData(',
              '   context: context,',
              '   text: "this is content",',
              '   okText: "确认",',
              '   cancelText: "取消",',
              '   maskClosable: false,',
              '   onOk: () {},',
              '))'
            ]
          )
        ])
      )
    );
  }
}

class OwnCard extends StatelessWidget {

  final String title;
  final List<String> content;

  OwnCard({
    this.title,
    this.content
  });

  List<Widget> renderContent() {
    List<Widget> _content = [];

    content.forEach((text) {
      _content.add(Text(text,
        style: TextStyle(fontSize: 15.0, color: Colors.white)
      ));
    });
    return _content;
  }

  @override
  Widget build(BuildContext context) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: <Widget>[
      Row(
        mainAxisAlignment: MainAxisAlignment.start,
        children: <Widget>[Text(title, style: TextStyle(
          fontSize: 16.0,
          fontWeight: FontWeight.bold
        ))]
      ),
      SizedBox(height: 4.0),
      Container(
        color: Color(0xFF29b1b1),
        padding: EdgeInsets.all(16.0),
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: renderContent()
        )
      )
    ]);
  }
}