import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class MPinCodeTextField extends StatefulWidget {
  MPinCodeTextField({
    this.autofocus = false,
    this.pinLength = 4,
    this.boxWidth=45,
    this.boxHeight=60,
    this.decoration,
    this.textStyle,
    this.onChanged,
    this.onEditingComplete,
    this.onSubmitted,
    this.minSpace = 20,
    this.obscure = true
  }):super();

  final int pinLength;
  final double boxWidth;
  final double boxHeight;
  final InputDecoration decoration;
  final TextStyle textStyle;
  final bool autofocus;
  final ValueChanged<String> onChanged;
  final VoidCallback onEditingComplete;
  final ValueChanged<String> onSubmitted;
  final double minSpace;
  final bool obscure;
  
  @override
  State<MPinCodeTextField> createState() => _MPinCodeTextFieldState();
}

class _MPinCodeTextFieldState extends State<MPinCodeTextField> {
  
  List<FocusNode> _focusNodeList = <FocusNode>[];
  List<TextEditingController> _editingControllerList = <TextEditingController>[];

  String _fullText() {
    String text = '';
    _editingControllerList.forEach((TextEditingController controller) {
      text += controller.text ?? '';
    });
    return text;
  }

  @override
  void initState() {
    super.initState();
    this._assembleEditControllers();
    this._assembleFocusNodes();
  }

  void _assembleFocusNodes() {
    _focusNodeList.clear();
    for (int index = 0; index < widget.pinLength; index++) {
      FocusNode focusNode = FocusNode();
      // focusNode.addListener(_checkFocusStatus);
      _focusNodeList.add(focusNode);
    }
  }
  void _assembleEditControllers() {
    _editingControllerList.clear();
    for (int index = 0; index < widget.pinLength; index++) {
      TextEditingController textEditingController = TextEditingController();
      // textEditingController.addListener(_popText);
      _editingControllerList.add(textEditingController);
    }
  }

  void _moveNext(int index) {
      _focusNodeList[index].unfocus();
    if (index < widget.pinLength - 1) {
      FocusNode nextFocusNode = _focusNodeList[++index];
      nextFocusNode.requestFocus();
    } else {
      widget.onEditingComplete?.call();
      widget.onSubmitted?.call(_fullText());
    }
  }

  void _movePrevious(int index) {
    if (index > 0) {
      FocusNode previousNode = _focusNodeList[--index];
      previousNode.requestFocus();
    } else {
      _focusNodeList[0].unfocus();
      widget.onEditingComplete?.call();
    }
  }

  @override
  Widget build(BuildContext context) {
    List<Widget> children = <Widget>[];
    print(_focusNodeList[0]);
    for (int i = 0; i < widget.pinLength; i++) {
      FocusNode focusNode = _focusNodeList[i];
      TextEditingController editingController = _editingControllerList[i];
      children.add(
        Container(
          width: widget.boxWidth,
          height: widget.boxHeight,
          child: _MTraceDeleteTextField(
            focusNode: focusNode,
            controller: editingController,
            decoration: widget.decoration,
            textInputAction: (i != widget.pinLength - 1)
                ? TextInputAction.next
                : TextInputAction.done,
            keyboardType: TextInputType.number,
            textStyle: widget.textStyle,
            textAlign: TextAlign.center,
            autofocus: (i == 0) ? widget.autofocus : false,
            onChanged: (String text) {
              if (text.length == 1) {
                _moveNext(i);
                widget.onChanged?.call(text);
              }
            },
            onSubmitted: (String text) {
              _moveNext(i);
            },
            onDeleted: () {
              _movePrevious(i);
            },
          ),
        ),
      );
      if (i != widget.pinLength - 1) {
        children.add(SizedBox(
          width: widget.minSpace ?? 8,
        ));
      }
    }

    return Row(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.center,
      children: children,
    );
  }
}

/// detect backspace related issue: https://github.com/flutter/flutter/issues/14809
class _MTraceDeleteTextField extends StatefulWidget {
  _MTraceDeleteTextField(
      {Key key,
      this.controller,
      this.focusNode,
      this.decoration,
      this.keyboardType,
      this.textInputAction,
      this.textStyle,
      this.textAlign,
      this.autofocus,
      this.obscure,
      this.onChanged,
      this.onEditingComplete,
      this.onSubmitted,
      this.onDeleted,
      this.inputFormatters,
      this.enabled})
      : super(key: key);

  final TextEditingController controller;
  final FocusNode focusNode;
  final InputDecoration decoration;
  final TextInputType keyboardType;
  final TextInputAction textInputAction;
  final TextStyle textStyle;
  final TextAlign textAlign;
  final bool autofocus;
  final bool obscure;
  final ValueChanged<String> onChanged;
  final VoidCallback onEditingComplete;
  final ValueChanged<String> onSubmitted;
  final VoidCallback onDeleted;
  final List<TextInputFormatter> inputFormatters;
  final bool enabled;

  @override
  State<_MTraceDeleteTextField> createState() =>
      _MTraceDeleteTextFieldState();
}

class _MTraceDeleteTextFieldState extends State<_MTraceDeleteTextField> {
  FocusNode get _effectiveFocusNode => widget.focusNode;
  TextEditingController _controller = TextEditingController();
  final String _sign = '';
  final String _nullString = '';

  bool _flag = true;
  bool get _userValue => _flag;
  set _userValue(bool value) {
    if (_flag != value) {
      setState(() {
        _flag = value;
      });
    }
  }

  @override
  void initState() {
    super.initState();
    _effectiveFocusNode.addListener(() {
      if (_effectiveFocusNode.hasFocus) {
        if (_userValue) _userValue = false;
        _clearControllerState();
      }
    });
  }

  void _clearControllerState() {
    widget.controller.text = _nullString;
    _controller
      ..text = _sign
      ..selection = TextSelection.collapsed(offset: _controller.text.length);
    widget.onChanged?.call(_nullString);
  }

  bool _validPopText() {
    if (_controller.text != _sign) {
      return true;
    }
    return false;
  }

  void _setText(String text) {
    _controller.text = text;
    widget.controller.text = text;
  }

  @override
  void dispose() {
    _effectiveFocusNode?.dispose();
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    TextStyle textStyle = !_userValue ? TextStyle(color: Colors.black38) : widget.textStyle;
    return TextField(
      controller: _controller,
      focusNode: widget.focusNode,
      decoration: widget.decoration,
      keyboardType: widget.keyboardType,
      textInputAction: widget.textInputAction,
      style: textStyle,
      autofocus: widget.autofocus,
      obscureText: false,
      textAlign: widget.textAlign,
      textAlignVertical: TextAlignVertical.center,
      inputFormatters: [
        WhitelistingTextInputFormatter(RegExp("[0-9]")),
        LengthLimitingTextInputFormatter(2)
      ],
      showCursor: true,
      onChanged: (text) {
        if (text.isEmpty) {
          _setText(_nullString);
          _userValue = false;
          widget.onDeleted?.call();
        } else {
          String newText = _controller.text[0];
          _userValue = true;
          _setText(newText);
          widget.onChanged?.call(newText);
        }
      },
      onEditingComplete: widget.onEditingComplete,
      onSubmitted: (text) {
        if (_validPopText()) {
          widget.onSubmitted(text);
        } else {
          widget.onSubmitted(_nullString);
        }
      },
      enabled: widget.enabled,
    );
  }
}
